/*
lcdpcf8574 lib sample

copyright (c) Davide Gironi, 2013

Released under GPLv3.
Please refer to LICENSE file for licensing information.
*/

/*
 * Lungoci Mihaela-Elisabeta, grupa 331CC
 */

#include <avr/io.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "lcdpcf8574.h"
#include "i2cmaster.h"

#include "usart.h"

//plate stepper
#define	stepPin PD4            //Define Step pin
#define Enable PD5              //Define Enable pin
#define dirPin PD6             //Define Direction pin

// knife stepper
#define	stepPin PB4            //Define Step pin
#define Enable PB5              //Define Enable pin
#define dirPin PB6             //Define Direction pin

#define F_CPU	16000000UL

#define PIN(x) (*(&x - 2))    /* address of input register of port x */

void printLCD (char* buffer, int line, int charPos){
	uint8_t led = 0;
	lcd_led(led); //set led
	int i;
	
	for (i = 0; i < strlen(buffer); i++) {
		lcd_gotoxy(charPos, line);
		lcd_putc(buffer[i]);
		charPos++;
		if(charPos == 16) {
			charPos = 0;
			line++;
			line %=2;
			lcd_gotoxy(0, line);
			lcd_puts("                ");
		}			
		_delay_ms(2000);
	}
}


int main(void)
{
	PORTD = 1<<PD5;
	PORTB = 1<<PB5;
	
	sei();
	//init lcd
	lcd_init(LCD_DISP_ON_BLINK);
	
	//lcd go home
	lcd_home();
	
	int sliceNumber = 0;
	int beginWrite = 0;
	char* buffer = "Hello! I am CakeSamurai and I'llhelp you to get your cake sliced. \
	In order to   proceed you haveto tell me how  many slices you want. Now, enterthe \
	number of   slices, please. (eg. 12#)So, thelucky number is: ";
	
	printLCD(buffer, 0, beginWrite);
	_delay_ms(2000);
	int keypressed = 0;//integer for storing matrix value
	
	DDRA = 0x0F;//taking column pins as input and row pins as output
	_delay_ms(1);
	PORTA = 0xF0;// powering the row ins
	_delay_ms(1);
	
	while(1)
	{
		if (PINA != 0b00001111)//in any of column pins goes high execute the loop
		{
			_delay_ms(5);
			keypressed = PINA;	//taking the column value into integer
			DDRA ^= 0b11111111;	//making rows as inputs and columns as ouput
			_delay_ms(1);
			PORTA ^= 0b11111111;	//powering columns
			_delay_ms(1);
			
			keypressed |= PINA;	//taking row value and or ing it to column value
			
			if (keypressed == 0b11101110)
			{
				printLCD("1", 1, beginWrite++);	//if row1 and column1 is low show 1
				sliceNumber *= 10;
				sliceNumber += 1;
			}
			if (keypressed == 0b11101101)
			{
				printLCD("4", 1, beginWrite++);// if row1 and column2 is low show 4
				sliceNumber *= 10;
				sliceNumber += 4;
			}
			if (keypressed == 0b11101011)
			{
				printLCD("7", 1, beginWrite++);// if row1 and column3 is low show 7
				sliceNumber *= 10;
				sliceNumber += 7;
			}
			if (keypressed == 0b11100111)
			{
				printLCD("*", 1, beginWrite++);//if row1 and column4 is low show *
			}
			if (keypressed == 0b11011110)
			{
				printLCD("2", 1, beginWrite++);// if row2 and column1 is low show 2
				sliceNumber *= 10;
				sliceNumber += 2;
			}
			if (keypressed == 0b11011101)
			{
				printLCD("5", 1, beginWrite++);// if row2 and column2 is low show 5
				sliceNumber *= 10;
				sliceNumber += 5;
			}
			if (keypressed == 0b11011011)
			{
				printLCD("8", 1, beginWrite++);// if row2 and column3 is low show 8
				sliceNumber *= 10;
				sliceNumber += 8;
			}
			if (keypressed == 0b11010111)
			{
				printLCD("0", 1, beginWrite++);// if row2 and column4 is low show 0
				sliceNumber *= 10;
				sliceNumber += 0;
			}
			if (keypressed == 0b10111110)
			{
				printLCD("3", 1, beginWrite++);
				sliceNumber *= 10;
				sliceNumber += 3;
			}
			if (keypressed == 0b10111101)
			{
				printLCD("6", 1, beginWrite++);
				sliceNumber *= 10;
				sliceNumber += 6;
			}
			if (keypressed == 0b10111011)
			{
				printLCD("9", 1, beginWrite++);
				sliceNumber *= 10;
				sliceNumber += 9;
			}
			if (keypressed == 0b10110111)
			{
				printLCD("#", 1, beginWrite++);
				lcd_gotoxy(0, 0);
				lcd_puts("                 ");
				lcd_gotoxy(0, 1);
				lcd_puts("                 ");
				char s[10];
				itoa(sliceNumber, s, 10);
				printLCD("Slicing ", 0, 0);
				printLCD(s, 0, 8);
				
					int x,y;

					DDRD |= (1<<PD5) | (1<<PD6) | (1<<PD4);     // Configure PORTD5, PORTD6, PORTD4 as output
					DDRB |= (1<<PB5) | (1<<PB6) | (1<<PB4);     // Configure PORTD5, PORTD6, PORTD4 as output
					
					for(x = 0; x < sliceNumber; x++)		//full plate rotation 27 small rotation
					{
						PORTB &= ~(1<<PB5);                 // Enable driver
						PORTB |= (1<<PB6);                //Make PORTD6 high to rotate motor in clockwise direction
							
						for(y = 0; y < 1200; y++)	// 520 steps to cut the cake and the rest of them to get through it (force)
						{
							PORTB |= (1<<PB4);
							_delay_ms(100);

							PORTB &= ~(1<<PB4);
							_delay_ms(100);
						}
							
						PORTB &= ~(1<<PB6);                //Make PORTD6 high to rotate motor in anti-clockwise direction
							
						for(y = 0; y < 520; y++)	// 520 steps to get up the knife
						{
							PORTB |= (1<<PB4);
							_delay_ms(100);

							PORTB &= ~(1<<PB4);
							_delay_ms(100);
						}
						PORTB |= (1<<PB5);                 // Disable driver knife
							
						PORTD &= ~(1<<PD5);                 // Enable driver plate
						PORTD |= (1<<PD6);                //Make PORTD6 high to rotate motor in clockwise direction
							
						for(y = 0; y < 200*24.5/sliceNumber; y++)	//full motor rotation = 200 steps; full plate rotation = 200 * 24.5
						{
							PORTD |= (1<<PD4);
							_delay_ms(100);

							PORTD &=~(1<<PD4);
							_delay_ms(100);
						}
						PORTD |= (1<<PD5);                 // Disable driver
						_delay_ms(10000);
					}
					printLCD("I've finished!Doyou dare to take a bite?", 0, 0);
					break;
			}
			if (keypressed == 0b01111110)
			{
				printLCD("A", 1, beginWrite++);
			}
			if (keypressed == 0b01111101)
			{
				printLCD("B", 1, beginWrite++);
			}
			if (keypressed == 0b01111011)
			{
				printLCD("C", 1, beginWrite++);
			}
			if (keypressed == 0b01110111)
			{
				printLCD("D", 1, beginWrite++);
			}
			keypressed = 0;//after showing integer erasing the row column memory
			
			DDRA ^= 0b11111111; //shifting input and power port
			_delay_ms(1);
			PORTA ^= 0b11111111;//powering row pins of keypad
			_delay_ms(220);
		}
	}
	
	return 0;
}